/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.document.index.persistence.impl.db;

import com.topcoder.db.connectionfactory.DBConnectionException;
import com.topcoder.db.connectionfactory.DBConnectionFactory;

import com.topcoder.document.index.persistence.IndexPersistence;
import com.topcoder.document.index.persistence.IndexPersistenceException;
import com.topcoder.document.index.persistence.impl.DocumentIndexPersistenceUtil;
import com.topcoder.document.index.persistence.impl.PersistenceConfigurationException;

import java.sql.Connection;
import java.sql.SQLException;


/**
 * <p>
 * This is an abstraction of a typical data base driven persistence contract.
 * </p>
 *
 * <p>
 * This provides for getting a connection from the DBConnectionFactory as well as specifying transaction control.
 * This hides nicely the intricacies of configuration that would be used by all derived classes.
 * </p>
 *
 * <p>
 * Thread Safety : the concrete class that extends this class is required to be thread-safe, since all the methods
 * interact with the persistent storage.
 * However, all the remove, add and update methods, transaction is used, and if any error occurs during removal or
 * add, transaction will be rollbacked. It's clear that this class makes no guarantee of externally thread safe.
 * </p>
 *
 * @author AleaActaEst, TCSDEVELOPER
 * @version 1.0
 */
public abstract class AbstractDBIndexPersistence implements IndexPersistence {
    /**
     * <p>
     * Represent the connectionName property name.
     * </p>
     */
    private static final String CONNECTION_NAME = "connectionName";

    /**
     * <p>
     * This represents the connection factory from which we will obtain a pre-configured connection for our data
     * base access.
     * This is initialized in one of the constructors and once initialized cannot be changed.
     * Cannot be null.
     * <p>
     */
    private final DBConnectionFactory connectionFactory;

    /**
     * <p>
     * This represents a connection name (an alias) that is used to fetch a connection instance from the connection
     * factory.
     * This is initialized in one of the constructors and once initialized cannot be changed.
     * Can be null or an empty string, upon which it will try to use the default connection.
     * </p>
     */
    private final String connectionName;

    /**
     * <p>
     * An abstract constructor which will populate the connectionFactory and connectionName information from
     * configuration.
     * </p>
     *
     * @param namespace The namespace for connection-related properties.
     * @throws IllegalArgumentException if namespace param is null or empty.
     * @throws PersistenceConfigurationException if there are configuration issues encountered
     */
    protected AbstractDBIndexPersistence(String namespace)
        throws PersistenceConfigurationException {
        DocumentIndexPersistenceUtil.checkString(namespace, "namespace");

        this.connectionFactory = DBUtil.getDBConnectionFactory(namespace);
        this.connectionName = DocumentIndexPersistenceUtil.getString(namespace,
                CONNECTION_NAME, false);
    }

    /**
     * <p>
     * An abstract constructor which will populate the connectionFactory and connectionName information from input
     * parameters.
     * </p>
     *
     * @param connectionFactory DBConnectionFactory instance used to generate connections.
     * @param connectionName The name of the connection used for persistence operations, can be null.
     * @throws IllegalArgumentException if connectionFactory is null.
     */
    protected AbstractDBIndexPersistence(
        DBConnectionFactory connectionFactory, String connectionName) {
        DocumentIndexPersistenceUtil.checkNull(connectionFactory,
            "connectionFactory");

        this.connectionFactory = connectionFactory;
        this.connectionName = connectionName;
    }

    /**
     * <p>
     * Returns a new connection to be used for persistence. This method will obtain the connection from the
     * connectionFactory
     * using the connectionName alias.
     * If the connectionName is null or empty, then default default connection name in configuration file will be used.
     * </p>
     *
     * <p>
     * The returned connection has already been with their auto-commit set to false.
     * </p>
     *
     * @return the connection instance for database operation
     * @throws IndexPersistenceException If unable to obtain a Connection
     */
    protected Connection getConnection() throws IndexPersistenceException {
        Connection connection = null;

        try {
            // if the connectionName is null or empty, then the default connection of connectionFactory will be used.
            if ((connectionName == null)
                    || (connectionName.trim().length() == 0)) {
                connection = connectionFactory.createConnection();
            } else {
                connection = connectionFactory.createConnection(connectionName);
            }

            // we will use transaction while accessing to persistence
            connection.setAutoCommit(false);

            return connection;
        } catch (DBConnectionException e) {
            DBUtil.closeConnection(connection);
            throw new IndexPersistenceException("Failed to create connection with connectionFactory.", e);
        } catch (SQLException e) {
            DBUtil.closeConnection(connection);
            throw new IndexPersistenceException("Failed to create connection with connectionFactory.", e);
        }
    }

    /**
     * <p>
     * Starts a transaction on the passed in connection.
     * </p>
     *
     * <p>
     * Currently we do nothing here since auto commit is already false.
     * </p>
     *
     * @param connection the connection instance for database operation in the transaction.
     * @throws IllegalArgumentException if connection is null
     */
    protected void beginTransaction(Connection connection) {
        DocumentIndexPersistenceUtil.checkNull(connection, "connection");
    }

    /**
     * <p>
     * Commits a transaction on the passed in connection.
     * </p>
     *
     * <p>
     * Important, the connection will be closed finally.
     * </p>
     *
     * @param connection the connection instance for database operation in the transaction.
     * @throws IndexPersistenceException If an error occurs during this process, or if the Connection does not
     * support transactions.
     * @throws IllegalArgumentException if connection is null
     */
    protected void commitTransaction(Connection connection)
        throws IndexPersistenceException {
        DocumentIndexPersistenceUtil.checkNull(connection, "connection");

        try {
            connection.commit();
        } catch (SQLException e) {
            throw new IndexPersistenceException("Failed to commit transaction.", e);
        } finally {
            DBUtil.closeConnection(connection);
        }
    }

    /**
     * <p>
     * Rolls back the transaction.
     * </p>
     *
     * <p>
     * Important, the connection will be closed finally.
     * </p>
     *
     * @param connection the connection instance for database operation in the transaction.
     * @throws IndexPersistenceException If an error occurs during this process, or if the Connection does not
     * support transactions.
     * @throws IllegalArgumentException if connection is null
     */
    protected void rollbackTransaction(Connection connection)
        throws IndexPersistenceException {
        DocumentIndexPersistenceUtil.checkNull(connection, "connection");

        try {
            connection.rollback();
        } catch (SQLException e) {
            throw new IndexPersistenceException("Failed to rollback transaction.", e);
        } finally {
            DBUtil.closeConnection(connection);
        }
    }
}
