/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.document.index.persistence.impl;

import java.util.Set;
import com.topcoder.document.index.CollectionIndex;
import com.topcoder.document.index.DocumentIndex;
import com.topcoder.document.index.IndexManager;
import com.topcoder.document.index.persistence.IndexPersistence;
import com.topcoder.document.index.persistence.TestHelper;
import com.topcoder.document.index.persistence.impl.db.FastAccessDBIndexPersistence;
import com.topcoder.document.index.persistence.impl.db.SimpleAccessDBIndexPersistence;
import com.topcoder.document.index.persistence.impl.xml.XmlIndexPersistence;
import com.topcoder.document.index.wordsource.WordSource;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

/**
 * <p>
 * Component demonstration for Document Indexer Persistence Component.
 * </p>
 *
 * <p>
 * The Document Indexer Persistence component implements the persistence layer as required by the Document
 * Indexer component. The pluggable framework allows different persistence mechanisms to be used.
 * </p>
 *
 * <p>
 * This demo is separated into four parts.
 * </p>
 *
 * <p>
 * The first part demonstrates the usage of how to create an IndexPersistence instance using configuration files.
 * </p>
 *
 * <p>
 * The second part demonstrates the usage of how to create an IndexPersistence instance using arguments.
 * </p>
 *
 * <p>
 * The third part demonstrates the usage of how to perform persistence operation in manager-base way.
 * </p>
 *
 * <p>
 * The fourth part demonstrates the usage of how to perform persistence operation in direct-api way.
 * </p>
 *
 * @author TCSDEVELOPER
 * @version 1.0
 *
 */
public class Demo extends TestCase {
    /**
     * Return all tests.
     *
     * @return all tests
     */
    public static Test suite() {
        return new TestSuite(Demo.class);
    }

    /**
     * <p>
     * Demonstrates the usage of how to create an IndexPersistence instance using configuration files.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testCreationUsingconfigurationFileDemo() throws Exception {
        // set up namespaces
        TestHelper.loadMultipleXMLConfig(TestHelper.DB_FACTORY_NAMESPACE, TestHelper.DB_FACTORY_CONFIGFILE);
        TestHelper.loadSingleXMLConfig(TestHelper.SIMPLE_ACCESS_NAMESPACE, TestHelper.SIMPLE_ACCESS_CONFIGFILE);
        TestHelper.loadSingleXMLConfig(TestHelper.FAST_ACCESS_NAMESPACE, TestHelper.FAST_ACCESS_CONFIGFILE);
        TestHelper.loadSingleXMLConfig(TestHelper.XML_INDEX_NAMESPACE, TestHelper.XML_INDEX_CONFIGFILE);

        // create a simple access database based persistence
        new SimpleAccessDBIndexPersistence(TestHelper.SIMPLE_ACCESS_NAMESPACE);

        // create a fast access database based persistence
        new FastAccessDBIndexPersistence(TestHelper.FAST_ACCESS_NAMESPACE);

        // create a xml based persistence
        new XmlIndexPersistence(TestHelper.XML_INDEX_NAMESPACE);

        // clear namespaces
        TestHelper.clearConfigFile(TestHelper.DB_FACTORY_NAMESPACE);
        TestHelper.clearConfigFile(TestHelper.SIMPLE_ACCESS_NAMESPACE);
        TestHelper.clearConfigFile(TestHelper.FAST_ACCESS_NAMESPACE);
        TestHelper.clearConfigFile(TestHelper.XML_INDEX_NAMESPACE);
    }

    /**
     * <p>
     * Demonstrates the usage of how to create an IndexPersistence instance using arguments.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testCreationUsingArgumentsDemo() throws Exception {
        // set up namespace
        TestHelper.loadMultipleXMLConfig(TestHelper.DB_FACTORY_NAMESPACE, TestHelper.DB_FACTORY_CONFIGFILE);

        // create a simple access database based persistence
        new SimpleAccessDBIndexPersistence(TestHelper.getDBFactory(), TestHelper.SIMLE_ACCESS_CONNECTION_NAME);

        // create a fast access database based persistence
        new FastAccessDBIndexPersistence(TestHelper.getDBFactory(), TestHelper.FAST_ACCESS_CONNECTION_NAME);

        // create a xml based persistence
        new XmlIndexPersistence(1, 0, TestHelper.XML_PERSISTENCE_ROOT);

        // clear namespace
        TestHelper.clearConfigFile(TestHelper.DB_FACTORY_NAMESPACE);
    }

    /**
     * <p>
     * Demonstrates the usage of how to perform persistence operation in manager-base way.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testManagerBasedPersistenceUsageDemo() throws Exception {
        // Create a xml based persistence
        try {
            TestHelper.loadSingleXMLConfig(TestHelper.XML_INDEX_NAMESPACE, TestHelper.XML_INDEX_CONFIGFILE);
            IndexPersistence persistence = new XmlIndexPersistence(TestHelper.XML_INDEX_NAMESPACE);
            // pass it to the manager
            IndexManager manager = new IndexManager(persistence);

            WordSource source = TestHelper.getSimpleWordSource();
            String collectionId = "1234567898765";

            System.out.println("====ManageBasedPersistenceUsageDemo====");

            // create document index through manager
            DocumentIndex createdDocumentIndex = manager.createDocumentIndex(source);
            System.out.println("----createDocumentIndex----");
            displayDocumentIndex(createdDocumentIndex);

            // get document index through manager
            DocumentIndex gettedDocumentIndex = manager.getDocumentIndex(source.getId());
            System.out.println("----getDocumentIndex----");
            displayDocumentIndex(gettedDocumentIndex);

            // get document index use count through manager
            int useCount = manager.getDocumentUseCount(source.getId());
            System.out.println("----getDocumentUseCount----");
            System.out.println("Current use count is : " + useCount);

            // increase document index use count through manager
            manager.increaseDocumentUseCount(source.getId());
            System.out.println("----increaseDocumentUseCount----");
            System.out.println("After increasing use count is : " + manager.getDocumentUseCount(source.getId()));

            // decrease document index use count through manager
            manager.decreaseDocumentUseCount(source.getId());
            System.out.println("----decreaseDocumentUseCount----");
            System.out.println("After decreasing use count is : " + manager.getDocumentUseCount(source.getId()));

            // remove document index through manager
            manager.removeDocumentIndex(source.getId());

            // get all the indexed documents through manager
            Set set = manager.getIndexedDocuments();
            System.out.println("----getIndexedDocuments----");
            System.out.println("After removing there are " + set.size() + " documents indexed.");

            // create document collection through manager
            CollectionIndex createdCollectionIndex = manager.createCollectionIndex(collectionId);
            System.out.println("----createCollectionIndex----");
            displayCollectionIndex(createdCollectionIndex);

            // get document collection through manager
            CollectionIndex gettedCollectionIndex = manager.getCollectionIndex(collectionId);
            System.out.println("----getCollectionIndex----");
            displayCollectionIndex(gettedCollectionIndex);

            // update document collection through manager
            // the update operation is called in the addDocument() instance
            gettedCollectionIndex.addDocument(source);
            System.out.println("----updateCollectionIndex----");
            displayCollectionIndex(gettedCollectionIndex);

            // remove document collection through manager
            manager.removeCollectionIndex(collectionId);
        } finally {
            TestHelper.clearXmlSolutionFiles();
            TestHelper.clearConfigFile(TestHelper.XML_INDEX_NAMESPACE);
        }
    }

    /**
     * <p>
     * Demonstrates the usage of how to perform persistence operation in direct-api way.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testDirectPersistenceUsageDemo() throws Exception {
        try {
            // Create a xml based persistence
            TestHelper.loadSingleXMLConfig(TestHelper.XML_INDEX_NAMESPACE, TestHelper.XML_INDEX_CONFIGFILE);
            IndexPersistence persistence = new XmlIndexPersistence(TestHelper.XML_INDEX_NAMESPACE);
            IndexManager im = new IndexManager(persistence);

            WordSource source = TestHelper.getSimpleWordSource();
            String collectionId = "1234567898765";

            DocumentIndex documentIndex = new DocumentIndex(source);
            CollectionIndex collectionIndex = new CollectionIndex(im, collectionId);

            System.out.println("====DirectPersistenceUsageDemo====");

            // create document index using persistence implementation directly
            persistence.addDocumentIndex(documentIndex);
            System.out.println("----createDocumentIndex----");
            displayDocumentIndex(documentIndex);

            // get document index using persistence implementation directly
            DocumentIndex gettedDocumentIndex = persistence.getDocumentIndex(source.getId());
            System.out.println("----getDocumentIndex----");
            displayDocumentIndex(gettedDocumentIndex);

            // get document index use count using persistence implementation directly
            int useCount = persistence.getDocumentUseCount(source.getId());
            System.out.println("----getDocumentUseCount----");
            System.out.println("Current use count is : " + useCount);

            // increase document index use count using persistence implementation directly
            persistence.increaseDocumentUseCount(source.getId());
            System.out.println("----increaseDocumentUseCount----");
            System.out.println("After increasing use count is : " + persistence.getDocumentUseCount(source.getId()));

            // decrease document index use count using persistence implementation directly
            persistence.decreaseDocumentUseCount(source.getId());
            System.out.println("----decreaseDocumentUseCount----");
            System.out.println("After decreasing use count is : " + persistence.getDocumentUseCount(source.getId()));

            // remove document index using persistence implementation directly
            persistence.removeDocumentIndex(source.getId());

            // get all the indexed documents using persistence implementation directly
            Set set = persistence.getIndexedDocuments();
            System.out.println("----getIndexedDocuments----");
            System.out.println("After removing there are " + set.size() + " documents indexed.");

            // create document collection using persistence implementation directly
            persistence.addCollectionIndex(collectionIndex);
            System.out.println("----createCollectionIndex----");
            displayCollectionIndex(collectionIndex);

            // get document collection using persistence implementation directly
            CollectionIndex gettedCollectionIndex = persistence.getCollectionIndex(collectionId);
            System.out.println("----getCollectionIndex----");
            displayCollectionIndex(gettedCollectionIndex);

            // update document collection using persistence implementation directly
            // the update operation is called in the addDocument() instance
            collectionIndex.addDocument(source);
            System.out.println("----updateCollectionIndex----");
            displayCollectionIndex(collectionIndex);

            // remove document collection using persistence implementation directly
            persistence.removeCollectionIndex(collectionId);
        } finally {
            TestHelper.clearXmlSolutionFiles();
            TestHelper.clearConfigFile(TestHelper.XML_INDEX_NAMESPACE);
        }
    }

    /**
     * <p>
     * Displays the DocumentIndex instance.
     * </p>
     *
     * @param index DocumentIndex instance.
     */
    private void displayDocumentIndex(DocumentIndex index) {
        System.out.println("sourceClass : " + index.getWordSourceId().getSourceClassName());
        System.out.println("sourceIdentity : " + index.getWordSourceId().getSourceIdentity());
        System.out.println("sourceLocale : " + index.getWordSourceId().getSourceLocale());

        String[] delimiters = index.getWordSourceId().getDelimiters();

        System.out.print("delimiters : ");
        for (int i = 0; i < delimiters.length; i++) {
            if (i != 0) {
                System.out.print(", ");
            }
            System.out.print(DocumentIndexPersistenceUtil.hexCode(delimiters[i]));
        }
        System.out.println();
        System.out.println("wordsSize : " + index.getWords().size());
    }

    /**
     * <p>
     * Displays the CollectionIndex instance.
     * </p>
     *
     * @param index CollectionIndex instance.
     */
    private void displayCollectionIndex(CollectionIndex index) {
        System.out.println("collectionId : " + index.getId());
        System.out.println("associatiedDocumentsSize : " + index.getAllDocumentIds().size());
    }
}