/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.document.index.persistence.impl.db;

import com.topcoder.db.connectionfactory.DBConnectionFactory;

import com.topcoder.document.index.CollectionIndex;
import com.topcoder.document.index.DocumentIndex;
import com.topcoder.document.index.IndexManager;
import com.topcoder.document.index.persistence.TestHelper;
import com.topcoder.document.index.persistence.impl.DocumentIndexPersistenceUtil;
import com.topcoder.document.index.wordsource.WordSourceId;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import java.sql.Connection;

import java.util.ArrayList;
import java.util.List;
import java.util.Locale;
import java.util.Map;


/**
 * <p>
 * Unit test cases for DBUtil.
 * </p>
 *
 * @author TCSDEVELOPER
 * @version 1.0
 */
public class DBUtilTests extends TestCase {
    /**
     * <p>
     * Setup test environment.
     * </p>
     *
     * @throws Exception to JUit.
     */
    protected void setUp() throws Exception {
        TestHelper.loadMultipleXMLConfig(TestHelper.DB_FACTORY_NAMESPACE,
            TestHelper.DB_FACTORY_CONFIGFILE);
    }

    /**
     * <p>
     * Tears down test environment.
     * </p>
     *
     * @throws Exception to JUit.
     */
    protected void tearDown() throws Exception {
        DBConnectionFactory dbFactory = TestHelper.getDBFactory();
        TestHelper.clearSimpleSolutionTables(dbFactory.createConnection(
                TestHelper.SIMLE_ACCESS_CONNECTION_NAME));

        TestHelper.clearConfigFile(TestHelper.DB_FACTORY_NAMESPACE);
    }

    /**
     * <p>
     * Return all tests.
     * </p>
     *
     * @return all tests
     */
    public static Test suite() {
        return new TestSuite(DBUtilTests.class);
    }

    /**
     * <p>
     * Tests DBUtil#closeConnection(Connection) for accuracy.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testCloseConnection() throws Exception {
        Connection conn = TestHelper.getDBFactory().createConnection(TestHelper.SIMLE_ACCESS_CONNECTION_NAME);

        DBUtil.closeConnection(conn);

        assertTrue("Failed to close connection.", conn.isClosed());
    }

    /**
     * <p>
     * Tests DBUtil#removeDocumentIndex(Connection,String,boolean) and
     * DBUtil#insertDocument(Connection,DocumentIndex,String) for accuracy.
     * In this test case, queryDocument() is used to validate whether the insert and remove action
     * success or not.
     * </p>
     *
     * @throws Exception to JUit.
     */
    public void testInsertRemoveDocumentIndex() throws Exception {
        Connection conn = TestHelper.getDBFactory().createConnection(TestHelper.SIMLE_ACCESS_CONNECTION_NAME);

        try {
            DocumentIndex documentIndex = new DocumentIndex(TestHelper.getSimpleWordSource());
            String documentIndexId = DocumentIndexPersistenceUtil.getDigestBasedId(documentIndex.getWordSourceId());

            DBUtil.insertDocument(conn, documentIndex, documentIndexId);

            assertNotNull("Failed to insert document.",
                DBUtil.queryDocument(conn, documentIndexId));

            DBUtil.removeDocumentIndex(conn, documentIndexId, true);

            assertNull("Failed to remove document.",
                DBUtil.queryDocument(conn, documentIndexId));
        } finally {
            DBUtil.closeConnection(conn);
        }
    }

    /**
     * <p>
     * Tests DBUtil#removeCollectionIndex(Connection,String,List)
     * and DBUtil#insertCollectionIndex(Connection,CollectionIndex) for accuracy.
     * In this test case, getCollectionIndex() is used to validate whether the insert and remove action
     * success or not.
     * </p>
     *
     * @throws Exception to JUit.
     */
    public void testInsertRemoveCollectionIndex() throws Exception {
        Connection conn = TestHelper.getDBFactory().createConnection(TestHelper.SIMLE_ACCESS_CONNECTION_NAME);

        try {
            SimpleAccessDBIndexPersistence persistence = new SimpleAccessDBIndexPersistence(TestHelper.getDBFactory(),
                    TestHelper.SIMLE_ACCESS_CONNECTION_NAME);
            IndexManager im = new IndexManager(persistence);

            CollectionIndex collectionIndex = new CollectionIndex(im,
                    "topcoder_collection");

            DBUtil.insertCollectionIndex(conn, collectionIndex);

            assertNotNull("Failed to insert collection.",
                DBUtil.getCollectionIndex(conn, collectionIndex.getId(), true));

            DBUtil.removeCollectionIndex(conn, collectionIndex.getId(),
                new ArrayList());

            assertNotNull("Failed to remove collection.",
                DBUtil.getCollectionIndex(conn, collectionIndex.getId(), true));
        } finally {
            DBUtil.closeConnection(conn);
        }
    }

    /**
     * <p>
     * Tests DBUtil#getIndexedDocuments(Connection) for accuracy.
     * </p>
     *
     * @throws Exception to JUit.
     */
    public void testGetIndexedDocuments() throws Exception {
        Connection conn = TestHelper.getDBFactory().createConnection(TestHelper.SIMLE_ACCESS_CONNECTION_NAME);

        try {
            DocumentIndex documentIndex = new DocumentIndex(TestHelper.getSimpleWordSource());
            String documentIndexId = DocumentIndexPersistenceUtil.getDigestBasedId(documentIndex.getWordSourceId());

            DBUtil.insertDocument(conn, documentIndex, documentIndexId);

            assertEquals("Expects one element contained.", 1,
                DBUtil.getIndexedDocuments(conn).size());
        } finally {
            DBUtil.closeConnection(conn);
        }
    }

    /**
     * <p>
     * Tests DBUtil#getWordSourceId(Connection,String) for accuracy.
     * The document record is inserted by calling insertDocument() and then getWordSourceId() is used
     * to retrieve the record information and validate the result.
     * </p>
     *
     * @throws Exception to JUit.
     */
    public void testGetWordSourceId() throws Exception {
        Connection conn = TestHelper.getDBFactory().createConnection(TestHelper.SIMLE_ACCESS_CONNECTION_NAME);

        try {
            DocumentIndex documentIndex = new DocumentIndex(TestHelper.getSimpleWordSource());
            String documentIndexId = DocumentIndexPersistenceUtil.getDigestBasedId(documentIndex.getWordSourceId());

            DBUtil.insertDocument(conn, documentIndex, documentIndexId);

            WordSourceId id = DBUtil.getWordSourceId(conn, documentIndexId);

            assertEquals("Failed to get correct WordSourceId instance.",
                "test_files/simple.txt", id.getSourceIdentity());

            assertEquals("Failed to get correct WordSourceId instance.", 12,
                id.getDelimiters().length);

            assertEquals("Failed to get correct WordSourceId instance.",
                "com.topcoder.document.index.wordsource.textfile.TextFileWordSource",
                id.getSourceClassName());

            assertEquals("Failed to get correct WordSourceId instance.",
                Locale.US, id.getSourceLocale());
        } finally {
            DBUtil.closeConnection(conn);
        }
    }

    /**
     * <p>
     * Tests DBUtil#isCollectionIndexPersisted(Connection,String) for accuracy.
     * </p>
     *
     * @throws Exception to JUit.
     */
    public void testIsCollectionIndexPersisted() throws Exception {
        Connection conn = TestHelper.getDBFactory().createConnection(TestHelper.SIMLE_ACCESS_CONNECTION_NAME);

        try {
            SimpleAccessDBIndexPersistence persistence = new SimpleAccessDBIndexPersistence(TestHelper.getDBFactory(),
                    TestHelper.SIMLE_ACCESS_CONNECTION_NAME);
            IndexManager im = new IndexManager(persistence);

            CollectionIndex collectionIndex = new CollectionIndex(im,
                    "topcoder_collection");

            DBUtil.insertCollectionIndex(conn, collectionIndex);

            assertTrue("Failed to query collection record.",
                DBUtil.isCollectionIndexPersisted(conn, "topcoder_collection"));
        } finally {
            DBUtil.closeConnection(conn);
        }
    }

    /**
     * <p>
     * Tests DBUtil#executeQuery(Connection,String,List) and
     * DBUtil#executeUpdate(Connection,String,List) for accuracy.
     * </p>
     *
     * @throws Exception to JUit.
     */
    public void testExecuteQueryUpdate() throws Exception {
        Connection conn = TestHelper.getDBFactory().createConnection(TestHelper.SIMLE_ACCESS_CONNECTION_NAME);

        try {
            // insert a record into database using executeUpdate() method.
            String insertSql = "INSERT INTO DOCUMENT_COLLECTION (COLLECTION_ID) VALUES (?)";
            List list = new ArrayList();
            list.add("123458");

            int row = DBUtil.executeUpdate(conn, insertSql, list);

            assertEquals("Expects only one row affected.", 1, row);

            // query the record inserted by executeUpdate() method
            String querySql = "SELECT COLLECTION_ID FROM DOCUMENT_COLLECTION";

            List result = DBUtil.executeQuery(conn, querySql, new ArrayList());

            assertEquals("Expects only one record returned.", 1, result.size());

            Map map = (Map) result.get(0);
            String id = (String) map.get("COLLECTION_ID");

            assertEquals("Executes equal collection id.", "123458", id);
        } finally {
            DBUtil.closeConnection(conn);
        }
    }
}
